/*
 * $RCSfile: StreamSegment.java,v $
 *
 * 
 * Copyright (c) 2005 Sun Microsystems, Inc. All  Rights Reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met: 
 * 
 * - Redistribution of source code must retain the above copyright 
 *   notice, this  list of conditions and the following disclaimer.
 * 
 * - Redistribution in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in 
 *   the documentation and/or other materials provided with the
 *   distribution.
 * 
 * Neither the name of Sun Microsystems, Inc. or the names of 
 * contributors may be used to endorse or promote products derived 
 * from this software without specific prior written permission.
 * 
 * This software is provided "AS IS," without a warranty of any 
 * kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND 
 * WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
 * EXCLUDED. SUN MIDROSYSTEMS, INC. ("SUN") AND ITS LICENSORS SHALL 
 * NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
 * USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
 * DERIVATIVES. IN NO EVENT WILL SUN OR ITS LICENSORS BE LIABLE FOR 
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
 * REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
 * INABILITY TO USE THIS SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES. 
 * 
 * You acknowledge that this software is not designed or intended for 
 * use in the design, construction, operation or maintenance of any 
 * nuclear facility. 
 *
 * $Revision: 1.1 $
 * $Date: 2005/02/11 05:01:20 $
 * $State: Exp $
 */
package com.sun.media.imageio.stream;

/**
 * A utility class representing a segment within a stream as a
 * <code>long</code> starting position and an <code>int</code>
 * length.
 */
public class StreamSegment {
    
    private long startPos = 0L;
    private int segmentLength = 0;

    /**
     * Constructs a <code>StreamSegment</code>.
     * The starting position and length are set to 0.
     */
    public StreamSegment() {}

    /**
     * Constructs a <code>StreamSegment</code> with a
     * given starting position and length.
     *
     * @param startPos The initial position of the segment.
     * @param segmentLength The length of the segment.
     */
    public StreamSegment(long startPos, int segmentLength) {
        this.startPos = startPos;
        this.segmentLength = segmentLength;
    }

    /**
     * Returns the starting position of the segment.
     *
     * @return The initial position of the segment.
     * @see #setStartPos
     */
    public final long getStartPos() {
        return startPos;
    }

    /**
     * Sets the starting position of the segment.
     *
     * @param startPos The initial position of the segment.
     * @see #getStartPos
     */
    public final void setStartPos(long startPos) {
        this.startPos = startPos;
    }

    /**
     * Returns the length of the segment.
     *
     * @return The length of the segment.
     * @see #setSegmentLength
     */
    public final int getSegmentLength() {
        return segmentLength;
    }

    /**
     * Sets the length of the segment.
     *
     * @param segmentLength The length of the segment.
     * @see #getSegmentLength
     */
    public final void setSegmentLength(int segmentLength) {
        this.segmentLength = segmentLength;
    }
}
